
/*
UDP Server:
------------

Server side

   1. Initialize SDL_net library
   2. Open a socket on a specified port
   3. Allocate memory for packets
   4. Wait for a packet from the client
   5. Do something with the packet
   6. Free the memory and quit 
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL_net.h>

int main(int argc, char *argv[])
{
  int quit = 0;
  int port;
  UDPsocket sd;    //socket descriptor
  UDPpacket *p;    //pointer to packet memory
  
  system("clear");

  if (argc == 2)
  {
    port = atoi(argv[1]);
    printf("\nAccepting connections on port: %d\n", port);
  }
  
  else
  {
    port = 4555;
    printf("\nNo port given...\nDefaulting to port 4555...\n");
  }
  
  printf("Initializing SDL_net...");  
  if (SDLNet_Init() < 0)    //init sdl_net
  {
    fprintf(stderr, "FAILED!\nSDLNet_Init: %s\n", SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n");
  
  // open socket on port given
  
  printf("Opening port %d for incoming connections...", port);
  if (!(sd = SDLNet_UDP_Open(port)))
  {
    fprintf(stderr, "FAILED!\nSDLNet_UDP_Open(port): %s\n", SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n");
  
  // allocate space for the packet
  printf("Allocating memory for packets...");
  if (!(p = SDLNet_AllocPacket(512)))
  {
    fprintf(stderr, "FAILED!\nSDLNet_AllocPacket(512): %s\n", SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n");
  
  printf("Connection open; waiting for packets...\n\n");
  while (!quit)
  {
    //wait for packet. SDLNet_UDP_Recv returns non-zero on packet recieve
    if (SDLNet_UDP_Recv(sd, p))
    {
      printf("Packet incoming!\n");
      printf("\tChan: %d\n", p->channel);
      printf("\tData: %s\n", (char *)p->data);
      printf("\t Len: %d\n", p->len);
      printf("\tMax Len: %d\n", p->maxlen);
      printf("\tStatus: %d\n", p->status);
      printf("\tAddress: %x %x\n\n", p->address.host, p->address.port);
      
      //if they sent the quit command
      if (!strcmp((char *)p->data, "quit"))
      {
        system("clear");
        printf("\nServer recieved \"quit\" signal from client!\n");
        quit = 1;
      }
    }
  }
  
  printf("Shutting down server...\n");
  printf("Freeing allocated memory for packets...");
  SDLNet_FreePacket(p);
  printf("Done!\nExiting SDLNet...");
  SDLNet_Quit();
  
  printf("Done!\nTerminating program normally...\n");
  return 0;

}
